var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

/* Libaries */
import React, { useContext, useEffect, useRef } from 'react';
import styled, { css } from 'styled-components';
/* Components */
import Box from './Box';
import GhostButton from './GhostButton';
import Overlay from './Overlay';
import { DialogContext, Actions } from './DialogProvider';
/* Theme */
import * as theme from '../theme';

// The number of milliseconds it takes to close the dialog
var DIALOG_CLOSE_TRANSITION_LENGTH = 80;

/* Styles */
var Container = styled.div.withConfig({
  displayName: 'DialogRoot__Container',
  componentId: 'nnrqbd-0'
})(['display:flex;align-items:center;padding:', ';justify-content:center;height:100%;width:100%;position:fixed;top:0;left:0;z-index:9;pointer-events:none;', ''], theme.SPACING_700, function (p) {
  return p.active && css(['pointer-events:auto;']);
});

var Panel = styled.div.withConfig({
  displayName: 'DialogRoot__Panel',
  componentId: 'nnrqbd-1'
})(['background-color:white;border-radius:', ';box-shadow:0 10px 38px -10px hsla(208,24%,7%,.35),0 10px 20px -15px hsla(208,24%,7%,.2);max-height:100%;min-height:100px;opacity:0;overflow:hidden;transition-duration:', 'ms;transition-property:opacity,transform;transition-timing-function:linear;transform:translateY(5px);width:100%;z-index:8;pointer-events:none;', ' ', ' @media screen and (min-width:', '){', ' ', '}@media screen and (min-width:', '){', ' ', '}@media screen and (min-width:', '){', ' ', '}@media screen and (min-width:', '){', ' ', '}', ' ', ''], theme.BORDERRADIUS_200, DIALOG_CLOSE_TRANSITION_LENGTH, function (p) {
  return p.size1 && css(['max-width:800px;']);
}, function (p) {
  return p.size2 && css(['max-width:400px;']);
}, theme.BREAKPOINT_100, function (p) {
  return p.bp1_size1 && css(['max-width:800px;']);
}, function (p) {
  return p.bp1_size2 && css(['max-width:400px;']);
}, theme.BREAKPOINT_200, function (p) {
  return p.bp2_size1 && css(['max-width:800px;']);
}, function (p) {
  return p.bp2_size2 && css(['max-width:400px;']);
}, theme.BREAKPOINT_300, function (p) {
  return p.bp3_size1 && css(['max-width:800px;']);
}, function (p) {
  return p.bp3_size2 && css(['max-width:400px;']);
}, theme.BREAKPOINT_400, function (p) {
  return p.bp4_size1 && css(['max-width:800px;']);
}, function (p) {
  return p.bp4_size2 && css(['max-width:400px;']);
}, function (p) {
  return p.expanded && css(['height:100%;']);
}, function (p) {
  return p.active && css(['opacity:1;pointer-events:auto;transform:translateY(0);']);
});

var DialogRoot = function DialogRoot(_ref) {
  var children = _ref.children;

  var _useContext = useContext(DialogContext),
      state = _useContext.state,
      dispatch = _useContext.dispatch;

  var panelRef = useRef(null);
  var activeDialog = state.dialogs[0] || {};
  var active = !!state.dialogs[0] && !state.closing;

  // Close handler needs to not remove children of dialog until the dialog hide transition has fully completed in order
  // to avoid disgusting content reflows onscreen during close transition
  var close = function close() {
    var handler = function handler() {
      dispatch({ type: Actions.CLOSE, dialog: activeDialog });

      // Handler cleans up after itself
      if (panelRef.current) {
        panelRef.current.removeEventListener('transitionend', handler);
      }
    };

    if (panelRef.current) {
      panelRef.current.addEventListener('transitionend', handler);
    }

    dispatch({ type: Actions.BEGIN_CLOSING });
  };

  var handleOverlayClick = function handleOverlayClick() {
    if (activeDialog.dismissable) {
      close();
    }
  };

  // Close dialog if escape key is pressed
  var handleKeyDown = function handleKeyDown(event) {
    if (event.keyCode === 27 && active && activeDialog.dismissable) {
      event.preventDefault();
      event.stopPropagation();
      close();
    }
  };

  useEffect(function () {
    document.addEventListener('keydown', handleKeyDown);
    return function () {
      return document.removeEventListener('keydown', handleKeyDown);
    };
  });

  var container = React.createElement(
    Container,
    null,
    React.createElement(
      Box,
      {
        position_fixed: true,
        pt_1: true,
        pr_1: true,
        top_0: true,
        right_0: true,
        style: {
          transition: 'opacity ' + DIALOG_CLOSE_TRANSITION_LENGTH + 'ms linear',
          opacity: active ? '1' : '0',
          pointerEvents: active ? 'auto' : 'none'
        }
      },
      activeDialog.dismissable && React.createElement(
        GhostButton,
        { size2: true, onClick: close },
        React.createElement(
          'svg',
          {
            xmlns: 'http://www.w3.org/2000/svg',
            width: '25',
            height: '25',
            viewBox: '0 0 25 25',
            fill: 'none',
            stroke: 'currentColor'
          },
          React.createElement('path', { d: 'M7.5 17.5L17.5 7.5', strokeLinecap: 'round', strokeLinejoin: 'round' }),
          React.createElement('path', { d: 'M17.5 17.5L7.5 7.5', strokeLinecap: 'round', strokeLinejoin: 'round' })
        )
      )
    ),
    React.createElement(
      Panel,
      _extends({
        active: active,
        onClick: function onClick(event) {
          return event.stopPropagation();
        }
      }, activeDialog.panelProps, {
        ref: panelRef
      }),
      activeDialog.children && React.createElement(activeDialog.children, { close: close })
    )
  );

  return typeof children === 'function' ? children({ active: active, onClick: handleOverlayClick, children: container }) : React.createElement(
    Overlay,
    { active: active, onClick: handleOverlayClick },
    container
  );
};

export default DialogRoot;