var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

/* Libraries */
import React, { useState } from 'react';
import PropTypes from 'prop-types';
import styled from 'styled-components';
import * as theme from './../theme/';

var Thumb = styled.input.attrs({
  type: 'range'
}).withConfig({
  displayName: 'Slider__Thumb',
  componentId: 'sc-1ls054j-0'
})(['appearance:none;background:transparent;cursor:pointer;display:block;padding:', ' 0;vertical-align:middle;width:100%;&::-webkit-slider-runnable-track{background-color:transparent;border-radius:9999px;height:3px;}&::-moz-range-track{background-color:transparent;border:none;border-radius:9999px;cursor:pointer;height:3px;width:100%;}&::-webkit-slider-thumb{appearance:none;background-color:white;border:none;border-radius:50%;box-shadow:inset 0 0 0 1px ', ';cursor:pointer;height:', ';margin-top:-6px;transition:transform 100ms ease;width:', ';}&::-moz-range-thumb{appearance:none;background-color:white;border:none;border-radius:50%;box-shadow:inset 0 0 0 1px ', ';cursor:pointer;height:', ';margin-top:-6px;transition:transform 100ms ease;width:', ';}&:hover{&::-webkit-slider-thumb{box-shadow:inset 0 0 0 1px ', ';}}&:focus{outline:none;&::-webkit-slider-thumb{box-shadow:inset 0 0 0 1px ', ';}&::-moz-range-thumb{box-shadow:inset 0 0 0 1px ', ';}}'], theme.SPACING_100, theme.GRAY_500, theme.SPACING_300, theme.SPACING_300, theme.GRAY_500, theme.SPACING_300, theme.SPACING_300, theme.GRAY_600, theme.BLUE_500, theme.BLUE_500);

var Track = styled.div.withConfig({
  displayName: 'Slider__Track',
  componentId: 'sc-1ls054j-1'
})(['background-color:', ';border-radius:9999px;height:3px;pointer-events:none;position:absolute;width:100%;z-index:-1;'], theme.GRAY_400);

var InnerTrack = styled.div.withConfig({
  displayName: 'Slider__InnerTrack',
  componentId: 'sc-1ls054j-2'
})(['background:', ';border-radius:9999px;height:100%;'], theme.BLUE_500);

var Input = styled.div.withConfig({
  displayName: 'Slider__Input',
  componentId: 'sc-1ls054j-3'
})(['align-items:center;display:flex;position:relative;']);

var Slider = function Slider(_ref) {
  var value = _ref.value,
      _ref$min = _ref.min,
      min = _ref$min === undefined ? 0 : _ref$min,
      _ref$max = _ref.max,
      max = _ref$max === undefined ? 0 : _ref$max,
      largeStep = _ref.largeStep,
      onKeyDown = _ref.onKeyDown,
      onChange = _ref.onChange,
      props = _objectWithoutProperties(_ref, ['value', 'min', 'max', 'largeStep', 'onKeyDown', 'onChange']);

  var percentage = (value - min) * 100 / (max - min || 100);
  var handleChange = function handleChange(event) {
    return onChange(parseInt(event.target.value, 10));
  };
  var handleKeyDown = function handleKeyDown(event) {
    onKeyDown(event);

    if (event.shiftKey) {
      if (event.keyCode === 37) {
        onChange(Math.max(min, parseInt(value, 10) - largeStep));
      } else if (event.keyCode === 39) {
        onChange(Math.min(max, parseInt(value, 10) + largeStep));
      }

      event.preventDefault();
    }
  };

  return React.createElement(
    Input,
    null,
    React.createElement(
      Track,
      null,
      React.createElement(InnerTrack, { style: { width: percentage + '%' } })
    ),
    React.createElement(Thumb, _extends({}, props, {
      value: value,
      onChange: handleChange,
      onKeyDown: handleKeyDown
    }))
  );
};

Slider.propTypes = process.env.NODE_ENV !== "production" ? {
  onKeyDown: PropTypes.func,
  onChange: PropTypes.func,
  largeStep: PropTypes.number,
  step: PropTypes.string,
  value: PropTypes.number
} : {};

Slider.defaultProps = {
  step: '1',
  largeStep: 10,
  min: 0,
  max: 100,
  onChange: function onChange() {
    return undefined;
  },
  onKeyDown: function onKeyDown() {
    return undefined;
  }
};

export var SliderWrapper = function SliderWrapper(_ref2) {
  var initialValue = _ref2.value;

  var _useState = useState(initialValue),
      value = _useState[0],
      setValue = _useState[1];

  return React.createElement(Slider, { value: value, onChange: setValue });
};

export default Slider;