var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

import React, { useEffect, useRef, useState, useLayoutEffect } from 'react';
import styled, { css } from 'styled-components';
import Text from './Text';

var ACTIVE_DELAY = 800;
var RECENTLY_ACTIVE_THRESHOLD = 800;

var StyledTooltipContainer = styled.div.withConfig({
  displayName: 'Tooltip__StyledTooltipContainer',
  componentId: 'sc-18wfzt1-0'
})(['z-index:1000;pointer-events:none;opacity:0;position:absolute;transition-delay:0ms;transition-property:opacity;', ''], function (props) {
  return props.active && css(['opacity:1;']);
});

var StyledTooltipPanel = styled.div.withConfig({
  displayName: 'Tooltip__StyledTooltipPanel',
  componentId: 'sc-18wfzt1-1'
})(['border-radius:4px;position:absolute;background:hsla(0,0%,0%,.8);padding:7px;']);

var StyledArrow = styled.div.withConfig({
  displayName: 'Tooltip__StyledArrow',
  componentId: 'sc-18wfzt1-2'
})(['position:relative;&::after{border:solid transparent;content:\'\';height:0;width:0;position:absolute;pointer-events:none;border-color:transparent;margin-left:0;border-width:5px;top:100%;left:50%;', ' ', ' ', ' ', '}'], function (props) {
  return props.direction === 'top' && css(['border-top-color:hsla(0,0%,0%,.8);']);
}, function (props) {
  return props.direction === 'right' && css(['border-right-color:hsla(0,0%,0%,.8);']);
}, function (props) {
  return props.direction === 'bottom' && css(['border-bottom-color:hsla(0,0%,0%,.8);']);
}, function (props) {
  return props.direction === 'left' && css(['border-left-color:hsla(0,0%,0%,.8);']);
});

var ARROW_HEIGHT = 10;
var ARROW_WIDTH = 10;

var getBounds = function getBounds(element) {
  var _element$getBoundingC = element.getBoundingClientRect(),
      left = _element$getBoundingC.left,
      top = _element$getBoundingC.top,
      right = _element$getBoundingC.right,
      bottom = _element$getBoundingC.bottom,
      width = _element$getBoundingC.width,
      height = _element$getBoundingC.height;

  return {
    left: left + window.scrollX,
    top: top + window.scrollY,
    right: right + window.scrollX,
    bottom: bottom + window.scrollY,
    width: width,
    height: height
  };
};

var roundCoordinates = function roundCoordinates(_ref) {
  var top = _ref.top,
      left = _ref.left;
  return {
    top: Math.round(top),
    left: Math.round(left)
  };
};

var getTooltipLayout = function getTooltipLayout(alignment, direction, tooltipBounds, targetBounds) {
  // Determine the center of the target, as the both the placement of the tooltip body and the arrow are relative to it
  var targetCenter = {
    left: targetBounds.left + targetBounds.width / 2,
    top: targetBounds.top + targetBounds.height / 2
  };

  // Values for the tooltip's left edge which place it at the following positions
  var left = {
    start: targetBounds.left,
    center: targetCenter.left - tooltipBounds.width / 2,
    end: targetBounds.right - tooltipBounds.width
  };

  // Values for the tooltip's top edge which place it at the following positions
  var top = {
    start: targetBounds.top,
    center: targetCenter.top - tooltipBounds.height / 2,
    end: targetBounds.bottom - tooltipBounds.height
  };

  // Values for top and left at which the tooltip must start in order for its inner edge to be positioned at the correct
  // distance from the target's outer edge
  var origin = {
    top: targetBounds.top - tooltipBounds.height - ARROW_HEIGHT,
    right: targetBounds.right + ARROW_WIDTH,
    bottom: targetBounds.bottom + ARROW_HEIGHT,
    left: targetBounds.left - tooltipBounds.width - ARROW_HEIGHT
  };

  var tooltipPositionCalculations = {
    top: {
      start: function start() {
        return { left: left.start, top: origin.top };
      },
      center: function center() {
        return { left: left.center, top: origin.top };
      },
      end: function end() {
        return { left: left.end, top: origin.top };
      }
    },
    right: {
      start: function start() {
        return { left: origin.right, top: top.start };
      },
      center: function center() {
        return { left: origin.right, top: top.center };
      },
      end: function end() {
        return { left: origin.right, top: top.end };
      }
    },
    bottom: {
      start: function start() {
        return { left: left.start, top: origin.bottom };
      },
      center: function center() {
        return { left: left.center, top: origin.bottom };
      },
      end: function end() {
        return { left: left.end, top: origin.bottom };
      }
    },
    left: {
      start: function start() {
        return { left: origin.left, top: top.start };
      },
      center: function center() {
        return { left: origin.left, top: top.center };
      },
      end: function end() {
        return { left: origin.left, top: top.end };
      }
    }
  };

  var arrowPositionCalculations = {
    top: function top() {
      return {
        left: targetCenter.left - ARROW_WIDTH / 2,
        top: targetBounds.top - ARROW_HEIGHT
      };
    },
    right: function right() {
      return {
        left: targetBounds.right,
        top: targetCenter.top - ARROW_HEIGHT / 2
      };
    },
    bottom: function bottom() {
      return {
        left: targetCenter.left - ARROW_WIDTH / 2,
        top: targetBounds.bottom
      };
    },
    left: function left() {
      return {
        left: targetBounds.left - ARROW_WIDTH,
        top: targetCenter.top - ARROW_HEIGHT / 2
      };
    }
  };

  return {
    tooltipPosition: roundCoordinates(tooltipPositionCalculations[direction][alignment]()),
    arrowPosition: roundCoordinates(arrowPositionCalculations[direction]())
  };
};

var Tooltip = function Tooltip() {
  var _useState = useState({
    // top | right | bottom | left
    alignment: 'bottom',
    // start | center | end
    direction: 'start',
    dirty: false,
    content: '',
    layout: {},
    target: null
  }),
      state = _useState[0],
      setState = _useState[1];

  var _useState2 = useState(false),
      recentlyActive = _useState2[0],
      setRecentlyActive = _useState2[1];

  var _useState3 = useState(false),
      active = _useState3[0],
      setActive = _useState3[1];

  var _useState4 = useState(null),
      activeTimeout = _useState4[0],
      setActiveTimeout = _useState4[1];

  var _useState5 = useState(null),
      unactiveTimeout = _useState5[0],
      setUnactiveTimeout = _useState5[1];

  var tooltip = useRef(null);
  var disableTooltip = function disableTooltip() {
    return setActive(false);
  };

  useEffect(function () {
    // Build mouseover listener
    var handleAttention = function handleAttention(event) {
      var target = event.target;
      if (!(target instanceof HTMLElement)) {
        setActive(false);
        return;
      }

      var _target$dataset = target.dataset,
          tooltipContent = _target$dataset.tooltipContent,
          tooltipDirection = _target$dataset.tooltipDirection,
          tooltipAlignment = _target$dataset.tooltipAlignment;

      // If two consecutive attention events occur to the same element, we can hide the tooltip

      if (!tooltipContent || target === state.target) {
        // If we were just active but are disabling now, set recentlyActive to true and set a timer to turn it off
        if (active) {
          setRecentlyActive(true);
          clearTimeout(unactiveTimeout);
          setUnactiveTimeout(setTimeout(function () {
            return setRecentlyActive(false);
          }, RECENTLY_ACTIVE_THRESHOLD));
        }

        // If we are not refocusing on the current target, there is a new element without tooltip data being focused,
        // so set target to null so we know to hide the tooltip. Otherwise, maintain the current target so that we know
        // we can hide the tooltip and keep it hidden until we receive a new target
        var targetBeingRefocused = target === state.target;
        setState(_extends({}, state, { target: targetBeingRefocused ? state.target : null }));
        setActive(false);
        clearTimeout(activeTimeout);
      } else {
        if (target.tabIndex === -1) {
          console.warn('🚨 tabIndex has not been defined on a component with data-tooltip-content attribute!');
        }

        // Otherwise, we should show it for this element
        setState({
          target: target,
          direction: tooltipDirection || 'start',
          alignment: tooltipAlignment || 'bottom',
          content: tooltipContent,
          layout: {},
          dirty: true
        });

        clearTimeout(activeTimeout);
        if (recentlyActive) {
          setActive(true);
        } else {
          setActiveTimeout(setTimeout(function () {
            return setActive(true);
          }, ACTIVE_DELAY));
        }
      }
    };

    window.addEventListener('mouseover', handleAttention);
    window.addEventListener('focusin', handleAttention);
    window.addEventListener('click', handleAttention);
    window.addEventListener('blur', disableTooltip);

    return function () {
      window.removeEventListener('mouseover', handleAttention);
      window.removeEventListener('focusin', handleAttention);
      window.removeEventListener('click', handleAttention);
      window.removeEventListener('blur', disableTooltip);
    };
  });

  useLayoutEffect(function () {
    var direction = state.direction,
        alignment = state.alignment,
        target = state.target;

    if (!state.target) return setActive(false);
    if (!active || !state.dirty) {
      return;
    }

    // We've just rendered the tooltip. If there is content, determine tooltip's new position based on that content:
    var newLayout = getTooltipLayout(alignment, direction, getBounds(tooltip.current), getBounds(target));

    setState(_extends({}, state, { layout: newLayout, dirty: false }));
  });

  var _state$layout = state.layout,
      layout = _state$layout === undefined ? {} : _state$layout;

  return React.createElement(
    StyledTooltipContainer,
    { active: active },
    React.createElement(StyledArrow, { direction: state.direction, style: _extends({}, layout.arrowPosition) }),
    React.createElement(
      StyledTooltipPanel,
      { ref: tooltip, style: _extends({}, layout.tooltipPosition) },
      React.createElement(
        Text,
        { nowrap: true, size2: true, color_white: true },
        state.content
      )
    )
  );
};

export default Tooltip;